
var RDFInspectorSettings = {
    mPrefs : null,
    mRDF : null,
    mLocalstore : null,
    mRes : {},
    init : function() {
        this.mPrefs = Components
            .classes['@mozilla.org/preferences-service;1']
            .getService(Components.interfaces.nsIPrefBranch);
        this.mRDF = Components
            .classes['@mozilla.org/rdf/rdf-service;1']
            .getService()
            .QueryInterface(Components.interfaces.nsIRDFService);
        this.mLocalstore = this.mRDF.GetDataSource('rdf:local-store');
        this.mRes.root = this.mRDF.GetResource('chrome://rdfinspector/content/rdfinspector.xul');
        this.mRes.namespaces = this.mRDF.GetResource('chrome://rdfinspector/content/rdfinspector.xul#namespace');
        this.mRes.prefix = this.mRDF.GetResource('chrome://rdfinspector/content/rdfinspector.xul#prefix');
        this.mRes.uri = this.mRDF.GetResource('chrome://rdfinspector/content/rdfinspector.xul#uri');
        this.mRes.enabled = this.mRDF.GetResource('chrome://rdfinspector/content/rdfinspector.xul#enabled');
    },
    get localstore() {
        if (!this.mLocalstore) this.init();
        return this.mLocalstore;
    },

    StringPref : function (aName, aValue) {
        if (!this.mLocalstore) this.init();
        if (aValue == null) {
        	if (this.mPrefs.getPrefType(aName) == this.mPrefs.PREF_STRING)
	            return this.mPrefs.getCharPref(aName);
	        else
            	return null;
        } else {
            return this.mPrefs.setCharPref(aName, aValue);
        }
    },

    BoolPref : function (aName, aValue) {
        if (!this.mLocalstore) this.init();
        if (aValue == null) {
	        if (this.mPrefs.getPrefType(aName) == this.mPrefs.PREF_BOOL)
	            return this.mPrefs.getBoolPref(aName);
	        else
	        	return null;
        } else {
            return this.mPrefs.setBoolPref(aName, aValue);
        }
    },

    SetNamespace : function(aPrefix, aURI, aEnabled) {
        var namespaceNode = this.mRDF.GetAnonymousResource();

        var uriRes = this.mRDF.GetResource(aURI);
        var prefixRes = this.mRDF.GetLiteral(aPrefix);
        var enabledRes = this.mRDF.GetLiteral(aEnabled.toString());

        this.localstore.Assert(this.mRes.root, this.mRes.namespaces, namespaceNode, true);
        this.localstore.Assert(namespaceNode, this.mRes.uri, uriRes, true);
        this.localstore.Assert(namespaceNode, this.mRes.prefix, prefixRes, true);
        this.localstore.Assert(namespaceNode, this.mRes.enabled, enabledRes, true);
    },

    RemoveNamespace : function(aPrefix) {
        var namespace = this.GetNamespaceResourceByPrefix(aPrefix);
        if (namespace == null)
            return null;

        var uri = this.localstore
            .GetTarget(namespace, this.mRes.uri, true)
            .QueryInterface(Components.interfaces.nsIRDFResource);
        var prefix = this.localstore
            .GetTarget(namespace, this.mRes.prefix, true)
            .QueryInterface(Components.interfaces.nsIRDFLiteral);
        var enabled = this.localstore
            .GetTarget(namespace, this.mRes.enabled, true)
            .QueryInterface(Components.interfaces.nsIRDFLiteral);
        try {
        this.localstore.Unassert(this.mRes.root, this.mRes.namespaces, namespace);
        this.localstore.Unassert(namespace, this.mRes.uri, uri);
        this.localstore.Unassert(namespace, this.mRes.prefix, prefix);
        this.localstore.Unassert(namespace, this.mRes.enabled, enabled);
        } catch (e) {
            dump("    Exception: " + e + "\n");
        }
    },

    HasNamespacePrefix : function(aPrefix) {
        var uri = this.GetNamespaceURI(aPrefix);
        if (uri)
            return true;
        else
            return false;
    },

    GetNamespaceResourceByURI : function(aURI) {
        var namespaceEnum = this.localstore
            .GetTargets(this.mRes.root, this.mRes.namespaces, true);
        while (namespaceEnum.hasMoreElements()) {
            var namespace = namespaceEnum.getNext();
            var uri = this.localstore
                .GetTarget(namespace, this.mRes.uri, true)
                .QueryInterface(Components.interfaces.nsIRDFResource);
            if (uri.Value == aURI) {
                return namespace;
            }
        }
        return null;
    },

    GetNamespaceResourceByPrefix : function(aPrefix) {
        var namespaceEnum = this.localstore
            .GetTargets(this.mRes.root, this.mRes.namespaces, true);
        while (namespaceEnum.hasMoreElements()) {
            var namespace = namespaceEnum.getNext();
            if (namespace instanceof Components.interfaces.nsIRDFResource) {
                var prefix = this.localstore
                    .GetTarget(namespace, this.mRes.prefix, true)
                    .QueryInterface(Components.interfaces.nsIRDFLiteral);
                if (prefix.Value == aPrefix) {
                    return namespace;
                }
            }
        }
        return null;
    },

    GetNamespacePrefix : function(aURI) {
        var namespace = this.GetNamespaceResourceByURI(aURI);
        if (namespace == null)
            return null;

        var prefix = this.localstore
            .GetTarget(namespace, this.mRes.prefix, true)
            .QueryInterface(Components.interfaces.nsIRDFLiteral);
        return prefix.Value;
    },

    GetNamespaceURI : function(aPrefix) {
        var namespace = this.GetNamespaceResourceByPrefix(aPrefix);
        if (namespace == null)
            return null;

        var uri = this.localstore
            .GetTarget(namespace, this.mRes.uri, true)
            .QueryInterface(Components.interfaces.nsIRDFResource);
        return uri.Value;
    },

    GetAllNamespaces : function() {
        var namespaceEnum = this.localstore
            .GetTargets(this.mRes.root, this.mRes.namespaces, true);
        var namespaces = new Array();
        while (namespaceEnum.hasMoreElements()) {
            var namespace = namespaceEnum.getNext();
            var prefix = this.localstore
                .GetTarget(namespace, this.mRes.prefix, true)
                .QueryInterface(Components.interfaces.nsIRDFLiteral);
            var uri = this.localstore
                .GetTarget(namespace, this.mRes.uri, true)
                .QueryInterface(Components.interfaces.nsIRDFResource);
            var enabled = this.localstore
                .GetTarget(namespace, this.mRes.enabled, true)
                .QueryInterface(Components.interfaces.nsIRDFLiteral);
            var enabledBool = false;
            if (enabled.Value == "true")
                enabledBool = true;
            var namespaceData = {
                prefix  : prefix.Value,
                uri     : uri.Value,
                enabled : enabledBool
            };
            namespaces.push(namespaceData);
        }
        return namespaces;
    },
};
